@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2015 Phusion Holding B.V.
+ *  Copyright (c) 2011-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -54,6 +54,7 @@
 #include <Core/ApplicationPool/Process.h>
 #include <Core/ApplicationPool/Options.h>
 #include <Core/SpawningKit/Factory.h>
+#include <Core/SpawningKit/Result.h>
 #include <Core/SpawningKit/UserSwitchingRules.h>
 #include <Shared/ApplicationPoolApiKey.h>
 
@@ -261,7 +262,8 @@ public:
 	void setupAttachOrDetachHook(const ProcessPtr process, HookScriptOptions &options) const;
 
 	unsigned int generateStickySessionId();
-	ProcessPtr createProcessObject(const Json::Value &json);
+	ProcessPtr createNullProcessObject();
+	ProcessPtr createProcessObject(const SpawningKit::Spawner &spawner, const SpawningKit::Result &spawnResult);
 	bool poolAtFullCapacity() const;
 	ProcessPtr poolForceFreeCapacity(const Group *exclude, boost::container::vector<Callback> &postLockActions);
 	void wakeUpGarbageCollector();
@@ -478,6 +480,8 @@ public:
 	bool garbageCollectable(unsigned long long now = 0) const;
 
 	void inspectXml(std::ostream &stream, bool includeSecrets = true) const;
+	void inspectPropertiesInAdminPanelFormat(Json::Value &result) const;
+	void inspectConfigInAdminPanelFormat(Json::Value &result) const;
 
 	/****** Out-of-band work ******/
 
