@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2017 Phusion Holding B.V.
+ *  Copyright (c) 2010-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -71,6 +71,8 @@
 #include <Constants.h>
 #include <InstanceDirectory.h>
 #include <FileDescriptor.h>
+#include <FileTools/PathSecurityCheck.h>
+#include <SystemTools/UserDatabase.h>
 #include <RandomGenerator.h>
 #include <BackgroundEventLoop.h>
 #include <LoggingKit/LoggingKit.h>
@@ -176,6 +178,7 @@ static void cleanup(const WorkingObjectsPtr &wo);
 
 /***** Functions *****/
 
+#if !BOOST_OS_MACOS
 static FILE *
 openOomAdjFileGetType(const char *mode, OomFileType &type) {
 	FILE *f = fopen("/proc/self/oom_score_adj", mode);
@@ -249,6 +252,7 @@ setOomScoreNeverKill() {
 
 	return oldScore;
 }
+#endif
 
 static void
 terminationHandler(int signo) {
@@ -533,7 +537,8 @@ usage() {
 	printf("Usage: " AGENT_EXE " watchdog <OPTIONS...>\n");
 	printf("Runs the " PROGRAM_NAME " watchdog.\n\n");
 	printf("The watchdog runs and supervises various " PROGRAM_NAME " agent processes,\n");
-	printf("namely the core and the UstRouter. Arguments marked with \"[A]\", e.g.\n");
+	printf("at this moment only the core (there was also a process called UstRouter but\n");
+	printf("it no longer exists). Arguments marked with \"[A]\", e.g.\n");
 	printf("--passenger-root and --log-level, are automatically passed to all supervised\n");
 	printf("agents, unless you explicitly override them by passing extra arguments to a\n");
 	printf("supervised agent specifically. You can pass arguments to a supervised agent by\n");
@@ -791,7 +796,9 @@ initializeBareEssentials(int argc, char *argv[], WorkingObjectsPtr &wo) {
 	 * for this watchdog. Note that the OOM score is inherited by child processes
 	 * so we need to restore it after each fork().
 	 */
+#if !BOOST_OS_MACOS
 	string oldOomScore = setOomScoreNeverKill();
+#endif
 
 	watchdogSchema = new Schema();
 	watchdogConfig = new ConfigKit::Store(*watchdogSchema);
@@ -804,7 +811,9 @@ initializeBareEssentials(int argc, char *argv[], WorkingObjectsPtr &wo) {
 
 	wo = boost::make_shared<WorkingObjects>();
 	workingObjects = wo.get();
+#if !BOOST_OS_MACOS
 	wo->extraConfigToPassToSubAgents["oom_score"] = oldOomScore;
+#endif
 }
 
 static void
@@ -907,15 +916,14 @@ lowerPrivilege() {
 	string userName = watchdogConfig->get("user").asString();
 
 	if (geteuid() == 0 && !userName.empty()) {
-		struct passwd *pwUser = getpwnam(userName.c_str());
-
-		if (pwUser == NULL) {
-			throw RuntimeException("Cannot lookup user information for user " +
-				userName);
+		OsUser osUser;
+		if (!lookupSystemUserByName(userName, osUser)) {
+			throw NonExistentUserException("Operating system user '" + userName
+				+ "' does not exist");
 		}
 
-		gid_t gid = pwUser->pw_gid;
-		string groupName = getGroupName(pwUser->pw_gid);
+		gid_t gid = osUser.pwd.pw_gid;
+		string groupName = lookupSystemGroupnameByGid(gid);
 
 		if (initgroups(userName.c_str(), gid) != 0) {
 			int e = errno;
@@ -929,11 +937,11 @@ lowerPrivilege() {
 				"to that of user '" + userName + "' and group '" + groupName +
 				"': cannot set group ID to " + toString(gid), e);
 		}
-		if (setuid(pwUser->pw_uid) != 0) {
+		if (setuid(osUser.pwd.pw_uid) != 0) {
 			int e = errno;
 			throw SystemException("Unable to lower " SHORT_PROGRAM_NAME " watchdog's privilege "
 				"to that of user '" + userName + "' and group '" + groupName +
-				"': cannot set user ID to " + toString(pwUser->pw_uid), e);
+				"': cannot set user ID to " + toString(osUser.pwd.pw_uid), e);
 		}
 		#ifdef __linux__
 			// When we change the uid, /proc/self/pid contents don't change owner,
@@ -944,8 +952,8 @@ lowerPrivilege() {
 			// http://man7.org/linux/man-pages/man5/proc.5.html (search for "dumpable")
 			prctl(PR_SET_DUMPABLE, 1);
 		#endif
-		setenv("USER", pwUser->pw_name, 1);
-		setenv("HOME", pwUser->pw_dir, 1);
+		setenv("USER", osUser.pwd.pw_name, 1);
+		setenv("HOME", osUser.pwd.pw_dir, 1);
 		setenv("UID", toString(gid).c_str(), 1);
 	}
 }
@@ -954,19 +962,52 @@ static void
 lookupDefaultUidGid(uid_t &uid, gid_t &gid) {
 	const string defaultUser = watchdogConfig->get("default_user").asString();
 	const string defaultGroup = watchdogConfig->get("default_group").asString();
-	struct passwd *userEntry;
 
-	userEntry = getpwnam(defaultUser.c_str());
-	if (userEntry == NULL) {
+	OsUser osUser;
+	if (!lookupSystemUserByName(defaultUser, osUser)) {
 		throw NonExistentUserException("Default user '" + defaultUser +
-			"' does not exist.");
+			"' does not exist");
 	}
-	uid = userEntry->pw_uid;
+	uid = osUser.pwd.pw_uid;
 
-	gid = lookupGid(defaultGroup);
-	if (gid == (gid_t) -1) {
+	OsGroup osGroup;
+	if (!lookupSystemGroupByName(defaultGroup, osGroup)) {
 		throw NonExistentGroupException("Default group '" + defaultGroup +
-			"' does not exist.");
+			"' does not exist");
+	}
+	gid = osGroup.grp.gr_gid;
+}
+
+static void
+warnIfInstanceDirVulnerable(const string &root) {
+	TRACE_POINT();
+
+	if (geteuid() != 0) {
+		return; // Passenger is not root, so no escalation.
+	}
+
+	vector<string> errors, checkErrors;
+	if (isPathProbablySecureForRootUse(root, errors, checkErrors)) {
+		if (!checkErrors.empty()) {
+			string message = "WARNING: unable to perform privilege escalation vulnerability detection:\n";
+			foreach (string line, checkErrors) {
+				message.append("\n - " + line);
+			}
+			P_WARN(message);
+		}
+	} else {
+		string message = "WARNING: potential privilege escalation vulnerability detected. " \
+			PROGRAM_NAME " is running as root, and part(s) of the " SHORT_PROGRAM_NAME
+			" instance directory (" + root + ") can be changed by non-root user(s):\n";
+		foreach (string line, errors) {
+			message.append("\n - " + line);
+		}
+		foreach (string line, checkErrors) {
+			message.append("\n - " + line);
+		}
+		message.append("\n\nPlease either fix up the permissions for the insecure paths, or use" \
+					   " a different location for the instance dir that can only be modified by root.");
+		P_WARN(message);
 	}
 }
 
@@ -998,6 +1039,10 @@ initializeWorkingObjects(const WorkingObjectsPtr &wo, InstanceDirToucherPtr &ins
 	if (watchdogConfig->get("integration_mode").asString() == "standalone") {
 		instanceOptions.properties["standalone_engine"] = watchdogConfig->get("standalone_engine").asString();
 	}
+
+	// check if path is safe
+	warnIfInstanceDirVulnerable(watchdogConfig->get("instance_registry_dir").asString());
+
 	wo->instanceDir = boost::make_shared<InstanceDirectory>(instanceOptions,
 		watchdogConfig->get("instance_registry_dir").asString());
 	wo->extraConfigToPassToSubAgents["instance_dir"] = wo->instanceDir->getPath();
