@@ -26,7 +26,7 @@
 
 #include <Crypto.h>
 #include <modp_b64.h>
-#include <Logging.h>
+#include <LoggingKit/LoggingKit.h>
 #include <string>
 #include <Utils/SystemTime.h>
 #include <Utils/StrIntUtils.h>
@@ -162,6 +162,7 @@ OSStatus Crypto::copyIdentityFromPKCS12File(const char *cPath,
 	return status;
 }
 
+#if PRE_HIGH_SIERRA
 void Crypto::killKey(const char *cLabel) {
 	SecIdentityRef id = NULL;
 	OSStatus status = lookupKeychainItem(cLabel, &id);
@@ -238,6 +239,7 @@ bool Crypto::preAuthKey(const char *path, const char *passwd, const char *cLabel
 		return false;
 	}
 }
+#endif
 
 bool Crypto::generateRandomChars(unsigned char *rndChars, int rndLen) {
 	FILE *fPtr = fopen("/dev/random", "r");
@@ -294,7 +296,7 @@ CFDataRef Crypto::genIV(size_t ivSize) {
 
 bool Crypto::getKeyBytes(SecKeyRef cryptokey, void **target, size_t &len) {
 	const CSSM_KEY *cssmKey;
-	CSSM_WRAP_KEY wrappedKey = {{0}};
+	CSSM_WRAP_KEY wrappedKey;
 
 	CSSM_CSP_HANDLE cspHandle = 0;
 	CSSM_CC_HANDLE ccHandle = 0;
@@ -322,6 +324,7 @@ bool Crypto::getKeyBytes(SecKeyRef cryptokey, void **target, size_t &len) {
 											&ccHandle);
 	if (error != CSSM_OK) { cssmPerror("CSSM_CSP_CreateSymmetricContext",error); }
 
+	memset(&wrappedKey, 0, sizeof(wrappedKey));
 	error = CSSM_WrapKey(ccHandle,
 				 creds,
 				 cssmKey,
