@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2014-2015 Phusion Holding B.V.
+ *  Copyright (c) 2014-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -41,8 +41,9 @@
 #include <utility>
 #include <string>
 #include <deque>
-#include <Logging.h>
+#include <LoggingKit/LoggingKit.h>
 #include <ServerKit/Context.h>
+#include <ServerKit/Config.h>
 #include <ServerKit/Errors.h>
 #include <ServerKit/Channel.h>
 #include <Utils/JsonUtils.h>
@@ -443,7 +444,7 @@ private:
 	 * mode, the reader is responsible for popping buffers. In the in-file mode, the writer
 	 * is responsible for popping buffers (and writing them to the file).
 	 */
-	boost::uint32_t bytesBuffered;
+	boost::uint32_t bytesBuffered; // number of bytes buffered in memory
 	MemoryKit::mbuf firstBuffer;
 	deque<MemoryKit::mbuf> moreBuffers;
 
@@ -527,14 +528,14 @@ private:
 	}
 
 	void callBuffersFlushedCallback() {
-		if (buffersFlushedCallback != NULL) {
+		if (buffersFlushedCallback) {
 			FBC_DEBUG("Calling buffersFlushedCallback");
 			buffersFlushedCallback(this);
 		}
 	}
 
 	void callDataFlushedCallback() {
-		if (dataFlushedCallback != NULL) {
+		if (dataFlushedCallback) {
 			FBC_DEBUG("Calling dataFlushedCallback");
 			dataFlushedCallback(this);
 		}
@@ -760,11 +761,12 @@ private:
 		P_ASSERT_EQ(readerState, RS_READING_FROM_FILE);
 		verifyInvariants();
 		MemoryKit::mbuf buffer(boost::move(readContext->buffer));
+		ssize_t result = readContext->req.result;
 		delete readContext;
 		inFileMode->readRequest = NULL;
 
-		if (readContext->req.result >= 0) {
-			int fd = readContext->req.result;
+		if (result >= 0) {
+			int fd = result;
 			unsigned int generation = this->generation;
 
 			assert(fd <= inFileMode->written);
@@ -791,7 +793,7 @@ private:
 				terminateReaderBecauseOfEOF();
 			}
 		} else {
-			int errcode = -readContext->req.result;
+			int errcode = -result;
 			setError(errcode, __FILE__, __LINE__);
 		}
 	}
@@ -1027,6 +1029,7 @@ private:
 
 		if (req->result == UV_ECANCELED) {
 			uv_fs_req_cleanup(req);
+			free(req);
 			delete fcContext;
 			return;
 		}
@@ -1041,6 +1044,7 @@ private:
 		}
 
 		uv_fs_req_cleanup(req);
+		free(req);
 		delete fcContext;
 	}
 
@@ -1336,12 +1340,18 @@ public:
 	 * buffers. For example, in case of FileBufferedFdSinkChannel, this event does
 	 * not imply that all the in-memory buffers have been written to the sink FD.
 	 * That's what `dataFlushedCallback` is for.
+	 *
+	 * N.B. this class intentionally doesn't manage buffersFlushedCallback in any way,
+	 * the user is responsible for ensuring correctness of the variable's content.
 	 */
 	Callback buffersFlushedCallback;
 	/**
 	 * Called when all buffered data (whether in-memory or on-disk) has been consumed
 	 * by the data callback. In case of FileBufferedFdSinkChannel, this means that all
 	 * buffered data has been written out to the sink FD.
+	 *
+	 * N.B. this class intentionally doesn't manage dataFlushedCallback in any way,
+	 * the user is responsible for ensuring correctness of the variable's content.
 	 */
 	Callback dataFlushedCallback;
 
@@ -1361,7 +1371,7 @@ public:
 
 	FileBufferedChannel(Context *context)
 		: Channel(context),
-		  config(&context->defaultFileBufferedChannelConfig),
+		  config(&context->config.fileBufferedChannelConfig),
 		  mode(IN_MEMORY_MODE),
 		  readerState(RS_INACTIVE),
 		  nbuffers(0),
@@ -1385,7 +1395,7 @@ public:
 	void setContext(Context *context) {
 		Channel::setContext(context);
 		if (config == NULL) {
-			config = &context->defaultFileBufferedChannelConfig;
+			config = &context->config.fileBufferedChannelConfig;
 		}
 	}
 
@@ -1543,6 +1553,11 @@ public:
 		return buffersFlushedCallback;
 	}
 
+	OXT_FORCE_INLINE
+	void clearBuffersFlushedCallback() {
+		buffersFlushedCallback = NULL;
+	}
+
 	OXT_FORCE_INLINE
 	void setBuffersFlushedCallback(Callback callback) {
 		buffersFlushedCallback = callback;
