@@ -85,12 +85,79 @@ ngx_str_null_terminate(ngx_str_t *str) {
     return result;
 }
 
-static void
-psg_variant_map_set_ngx_str(PsgVariantMap *m,
-    const char *name,
+static PsgJsonValue *
+psg_json_value_set_str_ne(PsgJsonValue *doc, const char *name,
+    const char *val, size_t size)
+{
+    if (size > 0) {
+        return psg_json_value_set_str(doc, name, val, size);
+    } else {
+        return NULL;
+    }
+}
+
+static PsgJsonValue *
+psg_json_value_set_ngx_str_ne(PsgJsonValue *doc, const char *name,
     ngx_str_t *value)
 {
-    psg_variant_map_set(m, name, (const char *) value->data, value->len);
+    return psg_json_value_set_str_ne(doc, name,
+        (const char *) value->data, value->len);
+}
+
+static PsgJsonValue *
+psg_json_value_set_ngx_flag(PsgJsonValue *doc, const char *name, ngx_flag_t value) {
+    if (value != NGX_CONF_UNSET) {
+        return psg_json_value_set_bool(doc, name, value);
+    } else {
+        return NULL;
+    }
+}
+
+static PsgJsonValue *
+psg_json_value_set_ngx_uint(PsgJsonValue *doc, const char *name, ngx_uint_t value) {
+    if (value != NGX_CONF_UNSET_UINT) {
+        return psg_json_value_set_uint(doc, name, value);
+    } else {
+        return NULL;
+    }
+}
+
+static PsgJsonValue *
+psg_json_value_set_strset(PsgJsonValue *doc, const char *name,
+    const ngx_str_t *ary, size_t count)
+{
+    PsgJsonValue *subdoc = psg_json_value_new_with_type(PSG_JSON_VALUE_TYPE_ARRAY);
+    PsgJsonValue *elem;
+    size_t i;
+
+    for (i = 0; i < count; i++) {
+        elem = psg_json_value_new_str((const char *) ary[i].data, ary[i].len);
+        psg_json_value_append_val(subdoc, elem);
+        psg_json_value_free(elem);
+    }
+
+    elem = psg_json_value_set_value(doc, name, -1, subdoc);
+    psg_json_value_free(subdoc);
+    return elem;
+}
+
+static PsgJsonValue *
+psg_json_value_set_with_autodetected_data_type(PsgJsonValue *doc,
+    const char *name, size_t name_len,
+    const char *val, size_t val_len,
+    char **error)
+{
+    PsgJsonValue *j_val, *result;
+
+    j_val = psg_autocast_value_to_json(val, val_len, error);
+    if (j_val == NULL) {
+        return NULL;
+    }
+
+    result = psg_json_value_set_value(doc, name, name_len, j_val);
+    psg_json_value_free(j_val);
+
+    return result;
 }
 
 /**
@@ -172,24 +239,24 @@ starting_watchdog_after_fork(void *_data, void *_params) {
  * fails (or -1 if that fails too).
  */
 static void
-open_log_file_for_after_forking(AfterForkData *data, PsgVariantMap *params) {
-    char *log_filename;
-    int   fd;
+open_log_file_for_after_forking(AfterForkData *data, PsgJsonValue *log_target) {
+    const PsgJsonValue *log_target_path;
+    int                 fd;
 
-    log_filename = psg_variant_map_get_optional(params, "log_file");
-    if (log_filename == NULL) {
+    log_target_path = psg_json_value_get(log_target, "path", (size_t) -1);
+    if (log_target_path == NULL) {
         ngx_log_error(NGX_LOG_ALERT, data->cycle->log, 0,
             "no " PROGRAM_NAME " log file configured, discarding log output");
         fd = -1;
     } else {
-        fd = open(log_filename, O_WRONLY | O_APPEND | O_CREAT, 0644);
+        fd = open(psg_json_value_as_cstr(log_target_path),
+            O_WRONLY | O_APPEND | O_CREAT, 0644);
         if (fd == -1) {
             ngx_log_error(NGX_LOG_ALERT, data->cycle->log, ngx_errno,
                 "could not open the " PROGRAM_NAME " log file for writing during Nginx startup,"
                 " some log lines might be lost (will retry from " SHORT_PROGRAM_NAME " core)");
         }
-        free(log_filename);
-        log_filename = NULL;
+        log_target_path = NULL;
     }
 
     if (fd == -1) {
@@ -216,7 +283,7 @@ open_log_file_for_after_forking(AfterForkData *data, PsgVariantMap *params) {
 }
 
 static ngx_int_t
-create_file(ngx_cycle_t *cycle, const u_char *filename, const u_char *contents, size_t len) {
+create_file(ngx_cycle_t *cycle, const u_char *filename, const u_char *contents, size_t len, uid_t uid, gid_t gid) {
     FILE  *f;
     int    ret;
     size_t total_written = 0, written;
@@ -230,6 +297,9 @@ create_file(ngx_cycle_t *cycle, const u_char *filename, const u_char *contents,
         do {
             ret = fchmod(fileno(f), S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
         } while (ret == -1 && errno == EINTR);
+        do {
+            ret = fchown(fileno(f), uid, gid);
+        } while (ret == -1 && errno == EINTR);
         do {
             written = fwrite(contents + total_written, 1,
                 len - total_written, f);
@@ -255,74 +325,71 @@ start_watchdog(ngx_cycle_t *cycle) {
     ngx_core_conf_t *core_conf;
     ngx_int_t        ret, result;
     ngx_uint_t       i;
-    ngx_str_t       *prestart_uris;
-    char           **prestart_uris_ary = NULL;
     AfterForkData    after_fork_data;
     ngx_keyval_t    *ctl = NULL;
-    PsgVariantMap   *params = NULL;
+    ngx_str_t        str;
+    PsgJsonValue    *w_config = NULL;
+    PsgJsonValue    *j_log_target;
     u_char  filename[NGX_MAX_PATH], *last;
     char   *passenger_root = NULL;
     char   *error_message = NULL;
+    passenger_autogenerated_main_conf_t *autogenerated_main_conf =
+        &passenger_main_conf.autogenerated;
 
     core_conf = (ngx_core_conf_t *) ngx_get_conf(cycle->conf_ctx, ngx_core_module);
     result    = NGX_OK;
-    params    = psg_variant_map_new();
+    w_config  = psg_json_value_new_with_type(PSG_JSON_VALUE_TYPE_OBJECT);
+    j_log_target = psg_json_value_new_with_type(PSG_JSON_VALUE_TYPE_OBJECT);
     after_fork_data.cycle = cycle;
     after_fork_data.log_fd = -1;
-    passenger_root = ngx_str_null_terminate(&passenger_main_conf.root_dir);
+    passenger_root = ngx_str_null_terminate(&autogenerated_main_conf->root_dir);
     if (passenger_root == NULL) {
         goto error_enomem;
     }
 
-    pp_app_type_detector_set_throttle_rate(pp_app_type_detector,
-        passenger_main_conf.stat_throttle_rate);
-
-    prestart_uris = (ngx_str_t *) passenger_main_conf.prestart_uris->elts;
-    prestart_uris_ary = calloc(sizeof(char *), passenger_main_conf.prestart_uris->nelts);
-    for (i = 0; i < passenger_main_conf.prestart_uris->nelts; i++) {
-        prestart_uris_ary[i] = ngx_str_null_terminate(&prestart_uris[i]);
-        if (prestart_uris_ary[i] == NULL) {
-            goto error_enomem;
-        }
+    if (autogenerated_main_conf->stat_throttle_rate != NGX_CONF_UNSET_UINT) {
+        pp_app_type_detector_set_throttle_rate(pp_app_type_detector,
+            autogenerated_main_conf->stat_throttle_rate);
     }
 
-    psg_variant_map_set_int    (params, "web_server_control_process_pid", getpid());
-    psg_variant_map_set        (params, "server_software", NGINX_VER, strlen(NGINX_VER));
-    psg_variant_map_set        (params, "server_version", NGINX_VERSION, strlen(NGINX_VERSION));
-    psg_variant_map_set_bool   (params, "multi_app", 1);
-    psg_variant_map_set_bool   (params, "load_shell_envvars", 1);
-    psg_variant_map_set_int    (params, "log_level", passenger_main_conf.log_level);
-    psg_variant_map_set_ngx_str(params, "file_descriptor_log_file", &passenger_main_conf.file_descriptor_log_file);
-    psg_variant_map_set_int    (params, "socket_backlog", passenger_main_conf.socket_backlog);
-    psg_variant_map_set_ngx_str(params, "data_buffer_dir", &passenger_main_conf.data_buffer_dir);
-    psg_variant_map_set_ngx_str(params, "instance_registry_dir", &passenger_main_conf.instance_registry_dir);
-    psg_variant_map_set_bool   (params, "disable_security_update_check", passenger_main_conf.disable_security_update_check);
-    psg_variant_map_set_ngx_str(params, "security_update_check_proxy", &passenger_main_conf.security_update_check_proxy);
-    psg_variant_map_set_bool   (params, "user_switching", passenger_main_conf.user_switching);
-    psg_variant_map_set_bool   (params, "show_version_in_header", passenger_main_conf.show_version_in_header);
-    psg_variant_map_set_bool   (params, "turbocaching", passenger_main_conf.turbocaching);
-    psg_variant_map_set_ngx_str(params, "default_user", &passenger_main_conf.default_user);
-    psg_variant_map_set_ngx_str(params, "default_group", &passenger_main_conf.default_group);
-    psg_variant_map_set_ngx_str(params, "default_ruby", &passenger_main_conf.default_ruby);
-    psg_variant_map_set_int    (params, "max_pool_size", passenger_main_conf.max_pool_size);
-    psg_variant_map_set_int    (params, "pool_idle_time", passenger_main_conf.pool_idle_time);
-    psg_variant_map_set_int    (params, "response_buffer_high_watermark", passenger_main_conf.response_buffer_high_watermark);
-    psg_variant_map_set_int    (params, "stat_throttle_rate", passenger_main_conf.stat_throttle_rate);
-    psg_variant_map_set_ngx_str(params, "analytics_log_user", &passenger_main_conf.analytics_log_user);
-    psg_variant_map_set_ngx_str(params, "analytics_log_group", &passenger_main_conf.analytics_log_group);
-    psg_variant_map_set_bool   (params, "union_station_support", passenger_main_conf.union_station_support);
-    psg_variant_map_set_ngx_str(params, "union_station_gateway_address", &passenger_main_conf.union_station_gateway_address);
-    psg_variant_map_set_int    (params, "union_station_gateway_port", passenger_main_conf.union_station_gateway_port);
-    psg_variant_map_set_ngx_str(params, "union_station_gateway_cert", &passenger_main_conf.union_station_gateway_cert);
-    psg_variant_map_set_ngx_str(params, "union_station_proxy_address", &passenger_main_conf.union_station_proxy_address);
-    psg_variant_map_set_strset (params, "prestart_urls", (const char **) prestart_uris_ary, passenger_main_conf.prestart_uris->nelts);
+    /* Note: WatchdogLauncher::start() sets a number of default values. */
+    psg_json_value_set_str_ne    (w_config, "web_server_module_version", PASSENGER_VERSION, strlen(PASSENGER_VERSION));
+    psg_json_value_set_str_ne    (w_config, "web_server_version", NGINX_VERSION, strlen(NGINX_VERSION));
+    psg_json_value_set_str_ne    (w_config, "server_software", NGINX_VER, strlen(NGINX_VER));
+    psg_json_value_set_bool      (w_config, "multi_app", 1);
+    psg_json_value_set_bool      (w_config, "default_load_shell_envvars", 1);
+    psg_json_value_set_value     (w_config, "config_manifest", -1, passenger_main_conf.manifest);
+    psg_json_value_set_ngx_uint  (w_config, "log_level", autogenerated_main_conf->log_level);
+    psg_json_value_set_ngx_str_ne(w_config, "file_descriptor_log_target", &autogenerated_main_conf->file_descriptor_log_file);
+    psg_json_value_set_ngx_uint  (w_config, "core_file_descriptor_ulimit", autogenerated_main_conf->core_file_descriptor_ulimit);
+    psg_json_value_set_ngx_uint  (w_config, "controller_socket_backlog", autogenerated_main_conf->socket_backlog);
+    psg_json_value_set_ngx_str_ne(w_config, "controller_file_buffered_channel_buffer_dir", &autogenerated_main_conf->data_buffer_dir);
+    psg_json_value_set_ngx_str_ne(w_config, "instance_registry_dir", &autogenerated_main_conf->instance_registry_dir);
+    psg_json_value_set_ngx_flag  (w_config, "security_update_checker_disabled", autogenerated_main_conf->disable_security_update_check);
+    psg_json_value_set_ngx_str_ne(w_config, "security_update_checker_proxy_url", &autogenerated_main_conf->security_update_check_proxy);
+    psg_json_value_set_ngx_flag  (w_config, "user_switching", autogenerated_main_conf->user_switching);
+    psg_json_value_set_ngx_flag  (w_config, "show_version_in_header", autogenerated_main_conf->show_version_in_header);
+    psg_json_value_set_ngx_flag  (w_config, "turbocaching", autogenerated_main_conf->turbocaching);
+    psg_json_value_set_ngx_str_ne(w_config, "default_user", &autogenerated_main_conf->default_user);
+    psg_json_value_set_ngx_str_ne(w_config, "default_group", &autogenerated_main_conf->default_group);
+    psg_json_value_set_ngx_str_ne(w_config, "default_ruby", &passenger_main_conf.default_ruby);
+    psg_json_value_set_ngx_uint  (w_config, "max_pool_size", autogenerated_main_conf->max_pool_size);
+    psg_json_value_set_ngx_uint  (w_config, "pool_idle_time", autogenerated_main_conf->pool_idle_time);
+    psg_json_value_set_ngx_uint  (w_config, "max_instances_per_app", autogenerated_main_conf->max_instances_per_app);
+    psg_json_value_set_ngx_uint  (w_config, "response_buffer_high_watermark", autogenerated_main_conf->response_buffer_high_watermark);
+    psg_json_value_set_ngx_uint  (w_config, "stat_throttle_rate", autogenerated_main_conf->stat_throttle_rate);
+    psg_json_value_set_ngx_str_ne(w_config, "admin_panel_url", &autogenerated_main_conf->admin_panel_url);
+    psg_json_value_set_ngx_str_ne(w_config, "admin_panel_auth_type", &autogenerated_main_conf->admin_panel_auth_type);
+    psg_json_value_set_ngx_str_ne(w_config, "admin_panel_username", &autogenerated_main_conf->admin_panel_username);
+    psg_json_value_set_ngx_str_ne(w_config, "admin_panel_password", &autogenerated_main_conf->admin_panel_password);
 
-    if (passenger_main_conf.core_file_descriptor_ulimit != NGX_CONF_UNSET_UINT) {
-        psg_variant_map_set_int(params, "core_file_descriptor_ulimit", passenger_main_conf.core_file_descriptor_ulimit);
+    if (autogenerated_main_conf->prestart_uris != NGX_CONF_UNSET_PTR) {
+        psg_json_value_set_strset(w_config, "prestart_urls", (ngx_str_t *) autogenerated_main_conf->prestart_uris->elts,
+            autogenerated_main_conf->prestart_uris->nelts);
     }
 
-    if (passenger_main_conf.log_file.len > 0) {
-        psg_variant_map_set_ngx_str(params, "log_file", &passenger_main_conf.log_file);
+    if (autogenerated_main_conf->log_file.len > 0) {
+        psg_json_value_set_ngx_str_ne(j_log_target, "path", &autogenerated_main_conf->log_file);
     } else if (cycle->new_log.file == NULL) {
         ngx_log_error(NGX_LOG_EMERG, cycle->log, 0, "Cannot initialize " PROGRAM_NAME
             " because Nginx is not configured with an error log file."
@@ -331,25 +398,41 @@ start_watchdog(ngx_cycle_t *cycle) {
         result = NGX_ERROR;
         goto cleanup;
     } else if (cycle->new_log.file->name.len > 0) {
-        psg_variant_map_set_ngx_str(params, "log_file", &cycle->new_log.file->name);
+        psg_json_value_set_ngx_str_ne(j_log_target, "path", &cycle->new_log.file->name);
     } else if (cycle->log->file->name.len > 0) {
-        psg_variant_map_set_ngx_str(params, "log_file", &cycle->log->file->name);
+        psg_json_value_set_ngx_str_ne(j_log_target, "path", &cycle->log->file->name);
     }
 
-    open_log_file_for_after_forking(&after_fork_data, params);
-    psg_variant_map_set_bool(params, "log_file_is_stderr",
-        after_fork_data.stderr_equals_log_file);
+    if (autogenerated_main_conf->ctl != NULL) {
+        ctl = (ngx_keyval_t *) autogenerated_main_conf->ctl->elts;
+        for (i = 0; i < autogenerated_main_conf->ctl->nelts; i++) {
+            psg_json_value_set_with_autodetected_data_type(w_config,
+                (const char *) ctl[i].key.data, ctl[i].key.len,
+                (const char *) ctl[i].value.data, ctl[i].value.len,
+                &error_message);
+            if (error_message != NULL) {
+                str.data = ctl[i].key.data;
+                str.len = ctl[i].key.len - 1;
+                ngx_log_error(NGX_LOG_EMERG, cycle->log, 0,
+                    "Error parsing ctl %V as JSON data: %s",
+                    &str, error_message);
+                result = NGX_ERROR;
+                goto cleanup;
+            }
+        }
+    }
 
-    ctl = (ngx_keyval_t *) passenger_main_conf.ctl->elts;
-    for (i = 0; i < passenger_main_conf.ctl->nelts; i++) {
-        psg_variant_map_set2(params,
-            (const char *) ctl[i].key.data, ctl[i].key.len - 1,
-            (const char *) ctl[i].value.data, ctl[i].value.len - 1);
+    open_log_file_for_after_forking(&after_fork_data, j_log_target);
+    if (after_fork_data.stderr_equals_log_file) {
+        psg_json_value_set_bool(j_log_target, "stderr", 1);
+    }
+    if (!psg_json_value_empty(j_log_target)) {
+        psg_json_value_set_value(w_config, "log_target", -1, j_log_target);
     }
 
     ret = psg_watchdog_launcher_start(psg_watchdog_launcher,
         passenger_root,
-        params,
+        w_config,
         starting_watchdog_after_fork,
         &after_fork_data,
         &error_message);
@@ -367,34 +450,26 @@ start_watchdog(ngx_cycle_t *cycle) {
                         "%s/web_server_info/control_process.pid",
                         psg_watchdog_launcher_get_instance_dir(psg_watchdog_launcher, NULL));
     *last = (u_char) '\0';
-    if (create_file(cycle, filename, (const u_char *) "", 0) != NGX_OK) {
+    if (create_file(cycle, filename, (const u_char *) "", 0, (uid_t) core_conf->user, (gid_t) -1) != NGX_OK) {
         result = NGX_ERROR;
         goto cleanup;
     }
-    do {
-        ret = chown((const char *) filename, (uid_t) core_conf->user, (gid_t) -1);
-    } while (ret == -1 && errno == EINTR);
     if (ret == -1) {
         result = NGX_ERROR;
         goto cleanup;
     }
 
 cleanup:
-    psg_variant_map_free(params);
+    psg_json_value_free(w_config);
+    psg_json_value_free(j_log_target);
     free(passenger_root);
     free(error_message);
-    if (prestart_uris_ary != NULL) {
-        for (i = 0; i < passenger_main_conf.prestart_uris->nelts; i++) {
-            free(prestart_uris_ary[i]);
-        }
-        free(prestart_uris_ary);
-    }
 
     if (after_fork_data.log_fd != -1) {
         close(after_fork_data.log_fd);
     }
 
-    if (result == NGX_ERROR && passenger_main_conf.abort_on_startup_error) {
+    if (result == NGX_ERROR && autogenerated_main_conf->abort_on_startup_error) {
         exit(1);
     }
 
@@ -455,7 +530,7 @@ pre_config_init(ngx_conf_t *cf)
  */
 static ngx_int_t
 init_module(ngx_cycle_t *cycle) {
-    if (passenger_main_conf.root_dir.len != 0 && !ngx_test_config) {
+    if (passenger_main_conf.autogenerated.root_dir.len != 0 && !ngx_test_config) {
         if (first_start) {
             /* Ignore SIGPIPE now so that, if the watchdog fails to start,
              * Nginx doesn't get killed by the default SIGPIPE handler upon
@@ -465,7 +540,7 @@ init_module(ngx_cycle_t *cycle) {
             first_start = 0;
         }
         if (start_watchdog(cycle) != NGX_OK) {
-            passenger_main_conf.root_dir.len = 0;
+            passenger_main_conf.autogenerated.root_dir.len = 0;
             return NGX_OK;
         }
         pp_current_cycle = cycle;
@@ -485,7 +560,7 @@ static ngx_int_t
 init_worker_process(ngx_cycle_t *cycle) {
     ngx_core_conf_t *core_conf;
 
-    if (passenger_main_conf.root_dir.len != 0 && !ngx_test_config) {
+    if (passenger_main_conf.autogenerated.root_dir.len != 0 && !ngx_test_config) {
         save_master_process_pid(cycle);
 
         core_conf = (ngx_core_conf_t *) ngx_get_conf(cycle->conf_ctx, ngx_core_module);
